package java.util.concurrent.atomic;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.Serializable;

public class LongAdder extends Striped64 implements Serializable
{
    private static final long serialVersionUID = 7249069246863182397L;
    
    public LongAdder() {
    }
    
    public void add(long n) {
        Striped64.Cell[] cells;
        if ((cells = this.cells) == null) {
            long base = this.base;
            if (this.casBase(base, base + n)) {
                return;
            }
        }
        boolean cas = true;
        int n2;
        Striped64.Cell cell;
        if (cells != null && (n2 = cells.length - 1) >= 0 && (cell = cells[getProbe() & n2]) != null) {
            Striped64.Cell cell2 = cell;
            long value = cell.value;
            if ((cas = cell2.cas(value, value + n)) != 0) {
                return;
            }
        }
        this.longAccumulate(n, null, (boolean)cas);
    }
    
    public void increment() {
        this.add(1L);
    }
    
    public void decrement() {
        this.add(-1L);
    }
    
    public long sum() {
        Striped64.Cell[] cells = this.cells;
        long base = this.base;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    base += cell.value;
                }
            }
        }
        return base;
    }
    
    public void reset() {
        Striped64.Cell[] cells = this.cells;
        this.base = 0L;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    cell.value = 0L;
                }
            }
        }
    }
    
    public long sumThenReset() {
        Striped64.Cell[] cells = this.cells;
        long base = this.base;
        this.base = 0L;
        if (cells != null) {
            for (int i = 0; i < cells.length; ++i) {
                Striped64.Cell cell;
                if ((cell = cells[i]) != null) {
                    base += cell.value;
                    cell.value = 0L;
                }
            }
        }
        return base;
    }
    
    @Override
    public String toString() {
        return Long.toString(this.sum());
    }
    
    @Override
    public long longValue() {
        return this.sum();
    }
    
    @Override
    public int intValue() {
        return (int)this.sum();
    }
    
    @Override
    public float floatValue() {
        return (float)this.sum();
    }
    
    @Override
    public double doubleValue() {
        return (double)this.sum();
    }
    
    private Object writeReplace() {
        return new LongAdder.SerializationProxy(this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Proxy required");
    }
    
    private static class SerializationProxy implements Serializable
    {
        private static final long serialVersionUID = 7249069246863182397L;
        private final long value;
        
        SerializationProxy(LongAdder longAdder) {
            this.value = longAdder.sum();
        }
        
        private Object readResolve() {
            LongAdder longAdder = new LongAdder();
            longAdder.base = this.value;
            return longAdder;
        }
    }
}
